@@ -1,6 +1,8 @@
 from functools import cache
 from json import loads
+from os import PathLike
 from os import stat
+from pathlib import Path
 from re import sub
 from typing import Callable
 from typing import Optional
@@ -13,7 +15,7 @@ from falocalrepo_database.tables import journals_table
 from falocalrepo_database.tables import submissions_table
 from falocalrepo_database.tables import users_table
 
-m_time: Callable[[str], float] = lambda f: int(stat(f).st_mtime)
+m_time: Callable[[Union[str, PathLike]], int] = lambda f: int(stat(f).st_mtime)
 default_sort: dict[str, str] = {submissions_table: "id", journals_table: "id", users_table: "username"}
 default_order: dict[str, str] = {submissions_table: "desc", journals_table: "desc", users_table: "asc"}
 
@@ -58,7 +60,7 @@ def load_submission(db_path: str, submission_id: int, _cache=None) -> Optional[E
 
 
 @cache
-def load_submission_files(db_path: str, submission_id: int, _cache=None) -> tuple[Optional[bytes], Optional[bytes]]:
+def load_submission_files(db_path: str, submission_id: int, _cache=None) -> tuple[Optional[Path], Optional[Path]]:
     with FADatabase(db_path) as db:
         return db.submissions.get_submission_files(submission_id)
 
@@ -137,7 +139,7 @@ def load_search(db_path: str, table: str, sort: str, order: str, params_: str =
 
 
 @cache
-def load_files_folder(db_path: str, _cache=None) -> str:
+def load_files_folder(db_path: str, _cache=None) -> Path:
     with FADatabase(db_path) as db:
         return db.files_folder
 
